#include <stdio.h>
#include <sys/timeb.h>
#include <time.h>
#include <errno.h>
#include <va/va.h>
#include <string.h>
#include <assert.h>

#include "acquisition.h"
#define min(x, y) (((x)>(y))?(y):(x))

//acquisition code below was copied from xiViewer
static SETUP	setup;
LPSETUP lps = &setup;
HANDLE	hMV = INVALID_HANDLE_VALUE;

void	setT0(void){
	lps->pcn0 = getcurms();
}
double	getT(void){
	lps->pcn = getcurms();
	return (double)(lps->pcn-lps->pcn0) / 1000;
}
void* acquisition(void* args)
{
HANDLE	hMVlocal = INVALID_HANDLE_VALUE;
MMGETBITMAPOPT GetBitmapOpt;
MMFILTER mmFilter;
LPWORD	pRawData;
DWORD	dwSerNum;
DWORD nIndex = 0;
MM40_RETURN	mvret;
int iPrevFrame = -1;
float tpfi0, tpfie, tpfr0, tpfre;

	XExposeEvent xev;
	Window win = (Window)args;
	xev.type = Expose;
	xev.display = x11_display;
	xev.window = win;
	xev.count = 0;
	XDestroyWindowEvent destr;
	destr.type = DestroyNotify;
	destr.display = x11_display;
	destr.window = win;
	destr.event = win;

	bzero(&GetBitmapOpt, sizeof(MMGETBITMAPOPT));
	bzero(&mmFilter, sizeof(MMFILTER));
	mmFilter.dwFilter		= MMF_SHTNEW;
	mmFilter.dwBPP			= 8;
	mmFilter.sharpStrength	= 0;
	mmFilter.gammaY			= 1.0;
	mmFilter.gammaC			= 0;

	lps->bRun = TRUE; 

	while (lps->bRun) {
		if (hMVlocal == INVALID_HANDLE_VALUE) {
			char* env = getenv("CAM_INDEX");
			if(env) {
				nIndex = atoi(env);
			}
			env = getenv("CAM_SN");
			if(env) {
				dwSerNum = atoi(env);
			} else {
				mm40GetDevice( nIndex, &dwSerNum );
			}
			mm40Initialize("", dwSerNum, &hMVlocal );
			if (  mmInitializeHardware( hMVlocal ) != MM40_OK ) { 
				lps->bRun = FALSE;
				break;
			}
			mmApplyCameraProfile( hMVlocal );
			mmSetGainVal(hMVlocal, lps->fGain);

			mmInitCMS(hMVlocal, (void*)"in.icc", (void*)"out.icc", CMS_INTENT_PERCEPTUAL, TRUE);

			if ((mvret = mmSetMode( hMVlocal, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy)) != MM40_OK){
				lps->bRun = FALSE;
				continue;
			}

			while(!mmIsSetModeComplete(hMVlocal));

			lps->fFPS = 100;
			lps->fFPS_sum = 100;
			mmGetModeTiming(hMVlocal, &tpfi0, &tpfie, &tpfr0, &tpfre);
			if (tpfie && tpfre) {
				lps->fFPS = (float)(1./tpfre);
				lps->fFPS_sum = lps->fFPS;
			}
			
			if (mmStartAcquisition( hMVlocal ) != MM40_OK ) {
				lps->bRun = FALSE;
				break;
			}
		}
		if ( hMVlocal != INVALID_HANDLE_VALUE ) {
			while ( 1 ) {
				if (!lps->bRun) break;
				mvret = mmWaitForField( hMVlocal, 1000 );	//$$$
				if ( mvret == MM40_OK ) break;
				if ( mvret == MM40_INVALID_HANDLE || mvret == MM40_ACQUISITION_STOPED ) {
					mmStopAcquisition( hMVlocal );
					mm40Uninitialize( hMVlocal );
					hMVlocal = INVALID_HANDLE_VALUE;
					break;
				}
				if ( mvret == MM40_NO_IMAGE ) continue;
			}
		}

		if (!lps->bRun) break;

		if ( hMVlocal == INVALID_HANDLE_VALUE )
			continue;

		pRawData=NULL;

		while( mmIsFieldReady(hMVlocal) == MM40_OK ) { // This 'while' is to grab all the fields standing in the queue
			if ( hMVlocal != INVALID_HANDLE_VALUE ) {

				if (mmAcquireField( hMVlocal, (HANDLE*)&pRawData ) != MM40_OK) {
					continue;
				}
			}
			if (pRawData==NULL) continue;
			if(!lps->color) {
				mmUpdateBayerFrame( hMVlocal, pRawData );
				mmRetrieveFrameBayerEx(hMVlocal, &lps->gf);
			} else {
				if(mmUpdateFrame( hMVlocal, pRawData ) != MM40_OK) {
					pRawData = NULL;
					continue;
				}
				mmRetrieveFrameEx(hMVlocal, &lps->gf);
			}
			lps->dwNframes++;
			if(lps->gf.curFN - (iPrevFrame + 1) < 1000) lps->dwLframes += (lps->gf.curFN - (iPrevFrame + 1));
			iPrevFrame = lps->gf.curFN;
		}

		if (!pRawData || !mmIsFrameComplete(hMVlocal)) {
			continue;
		}

		if(lps->color) {
			if (lps->bTakeWB) {
				lps->bTakeWB = FALSE;
				float f[4];
				double a;
				mmGetMeans(hMVlocal, 0, 0, lps->gf.curCX, lps->gf.curCY, f);
				a = (f[0]+f[1]/2+f[2]/2+f[3])/3;
				if(lps->gf.curMTX == MMM_BGGR){
					lps->Kb = a / f[0];
					lps->Kg = a / ((f[1]+f[2])/2);
					lps->Kr = a / f[3];
				}
				if(lps->gf.curMTX == MMM_RGGB){
					lps->Kr = a / f[0];
					lps->Kg = a / ((f[1]+f[2])/2);
					lps->Kb = a / f[3];
				}
				a = min(min(lps->Kr,lps->Kg),lps->Kb);
				if(a) {
					lps->Kr/=a; lps->Kg/=a; lps->Kb/=a;
				} else {
					lps->Kr = lps->Kg = lps->Kb = 1.0;
				}
			}
			mmFilter.rgbWhiteBal[0] = lps->Kr;
			mmFilter.rgbWhiteBal[1] = lps->Kg;
			mmFilter.rgbWhiteBal[2] = lps->Kb;
			mmSetupFilter(hMVlocal, &mmFilter);
			mmApplyDefaultColorCorrectionMatrix(hMVlocal);
			mmSetupCE(hMVlocal, 0, (1 << lps->gf.curBPP) - 1);

			// Process frame (applying filters and processing algorithms)
			if(mmProcessFrame(hMVlocal) != MM40_OK )
				continue;
			if(!lps->bmp.bp) {
				// Retrieving Bitmap
				bzero(&lps->bmp, sizeof(MMGETBITMAP));
				if(mmRetrieveBitmapAdv(hMVlocal, &lps->bmp, &GetBitmapOpt) != MM40_OK)
					lps->bmp.bp = 0;
				mmApplyPCS(hMVlocal, &lps->bmp);
			}
		}

		lps->bHaveData = TRUE;
		if (lps->bRun) {
			XSendEvent(x11_display, win, True, ExposureMask, (XEvent*)&xev);
			XFlush(x11_display);
		}
	}

	if (hMVlocal != INVALID_HANDLE_VALUE) {
		mmUninitCMS(hMVlocal);
		mmStopAcquisition( hMVlocal );
		mm40Uninitialize( hMVlocal );
	}
	mmFreeBitmap(0,&lps->bmp);
	XSendEvent(x11_display, win, True, StructureNotifyMask, (XEvent*)&destr);
	XFlush(x11_display);
	return 0;
}

void setupCam() {
	DWORD	devices;
	DWORD	dwSerial;
	DWORD nIndex = 0;

	bzero(lps, sizeof(SETUP));
	lps->bDrawImage = TRUE;
		lps->uiExposure = 10000;
		lps->uiMode = 0;
		lps->fGain = 0;
		lps->bDrawImage = TRUE;
		lps->zoom = 2;
		lps->Kr = lps->Kg = lps->Kb = 1.0;

	mm40GetNumberDevices("", &devices );
	if (devices) {
		char* env = getenv("CAM_INDEX");
		if(env) {
			nIndex = atoi(env);
		}
		env = getenv("CAM_SN");
		if(env) {
			dwSerial = atoi(env);
		} else {
			mm40GetDevice( nIndex, &dwSerial );
		}
		mm40Initialize("", dwSerial, &hMV );
		if (hMV == INVALID_HANDLE_VALUE) {
		} else {
			mmInitializeHardware( hMV );
			mmSetExtention(hMV, EXT_RAW_EN, 1);
			mmApplyCameraProfile( hMV );
			lps->dwNframes = 0;
			lps->dwPframes = 0;
			lps->dwLframes = 0;
			lps->dwTime = 0;
		}
	}
}

#define rgbtoy(R, G, B, Y) \
Y = (uint8_t)(( (  66 * (uint16_t)R + 129 * (uint16_t)G +  25 * (uint16_t)B + 128) >> 8) +  16);

#define rgbtoyuv(R, G, B, Y, U, V) \
rgbtoy(R, G, B, Y); \
U = (uint8_t)(( ( -38 * (int16_t)R -  74 * (int16_t)G + 112 * (int16_t)B + 128) >> 8) + 128); \
V = (uint8_t)(( ( 112 * (int16_t)R -  94 * (int16_t)G -  18 * (int16_t)B + 128) >> 8) + 128);

static bool has_derive = true;

int drawImage(VADisplay va_dpy, VASurfaceID surface_id, VAImageFormat *image_format, int surface_width, int surface_height,
		Window win, int win_width, int win_height) {
	VAImage surface_image;
	char *surface_p;
	VAStatus va_status;
	int i, j;

	if(!lps->show)
		return 1;
	if(has_derive) {
		va_status = vaDeriveImage(va_dpy, surface_id, &surface_image);
		if (va_status != VA_STATUS_SUCCESS) {
			fprintf(stderr, "%s:vaDeriveImage (%d) failed %d, will fallback to vaCreateImage/vaPutImage\n", __func__, __LINE__, va_status);
			has_derive = false;
		}
	}
	if(!has_derive) {
		va_status = vaCreateImage(va_dpy, image_format, surface_width, surface_height, &surface_image);
		CHECK_VASTATUS(va_status, "vaCreateImage", return 0);
	}
	if(surface_image.width < surface_width || surface_image.height < surface_height) {
		printf("Didn't get expected image size: %dx%d!\n", surface_image.width, surface_image.height);
		return 0;
	}
	if(surface_image.format.fourcc != VA_FOURCC_NV12 && surface_image.format.fourcc != VA_FOURCC_YV12) {
		printf("Didn't get expected image format: %x\n", surface_image.format.fourcc);
		return 0;
	}
	va_status = vaMapBuffer(va_dpy, surface_image.buf, (void**)&surface_p);
	CHECK_VASTATUS(va_status, "vaMapBuffer", vaDestroyImage(va_dpy,surface_image.image_id);return 0);
	if(lps->color) {
		unsigned char *rgb = (unsigned char*)lps->sbmp.bp;
		int uv_step = 1;
		if(surface_image.format.fourcc == VA_FOURCC_NV12)
			uv_step = 2;
#ifdef OPENCL
		cl_int err;
		cl_image_format format_bgra = { CL_BGRA, CL_UNORM_INT8 };
		cl_image_format format_y = { CL_A, CL_UNSIGNED_INT8 };
		cl_image_format format_uv = { CL_RG, CL_UNSIGNED_INT8 };
		cl_mem input, outputy, outputuv, outputu, outputv;
		if(!(input = clCreateImage2D(lps->context, CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR, &format_bgra, surface_width, surface_height, 0, rgb, &err))) {
			fprintf(stderr, "OpenCL error: could not create BGRA input image: %d!\n", err);
			return 0;
		}
		if(!(outputy = clCreateImage2D(lps->context, CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR, &format_y, surface_width, surface_height, surface_image.pitches[0],
						(unsigned char*)surface_p+surface_image.offsets[0], &err))) {
			fprintf(stderr, "OpenCL error: could not create Y output image: %d!\n", err);
			return 0;
		}
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			if(!(outputu = clCreateImage2D(lps->context, CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR, &format_y, surface_width/2, surface_height/2, surface_image.pitches[1],
							(unsigned char*)surface_p+surface_image.offsets[1], &err))) {
				fprintf(stderr, "OpenCL error: could not create U output image: %d!\n", err);
				return 0;
			}
			if(!(outputv = clCreateImage2D(lps->context, CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR, &format_y, surface_width/2, surface_height/2, surface_image.pitches[2],
							(unsigned char*)surface_p+surface_image.offsets[2], &err))) {
				fprintf(stderr, "OpenCL error: could not create V output image: %d!\n", err);
				return 0;
			}
		} else {
			if(!(outputuv = clCreateImage2D(lps->context, CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR, &format_uv, surface_width/2, surface_height/2, surface_image.pitches[1],
							(unsigned char*)surface_p+surface_image.offsets[1], &err))) {
				fprintf(stderr, "OpenCL error: could not create UV output image: %d!\n", err);
				return 0;
			}
		}
		err  = clSetKernelArg(lps->rgb2y, 0, sizeof(cl_mem), &input);
		err |= clSetKernelArg(lps->rgb2y, 1, sizeof(cl_mem), &outputy);
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			err |= clSetKernelArg(lps->rgb2u, 0, sizeof(cl_mem), &input);
			err |= clSetKernelArg(lps->rgb2u, 1, sizeof(cl_mem), &outputu);
			err |= clSetKernelArg(lps->rgb2v, 0, sizeof(cl_mem), &input);
			err |= clSetKernelArg(lps->rgb2v, 1, sizeof(cl_mem), &outputv);
		} else {
			err |= clSetKernelArg(lps->rgb2uv, 0, sizeof(cl_mem), &input);
			err |= clSetKernelArg(lps->rgb2uv, 1, sizeof(cl_mem), &outputuv);
		}
		if(err != CL_SUCCESS) {
			fprintf(stderr, "OpenCL error: failed to set kernel arguments!\n");
			return 0;
		}
		size_t global[2];
		global[0] = surface_width;
		global[1] = surface_height;
		if((err = clEnqueueNDRangeKernel(lps->commands, lps->rgb2y, 2, NULL, &global[0], NULL, 0, NULL, NULL)) != CL_SUCCESS) {
			fprintf(stderr, "OpenCL error: failed to execute rgb2u kernel: %d!\n", err);
			return 0;
		}
		global[0] /= 2;
		global[1] /= 2;
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			if((err = clEnqueueNDRangeKernel(lps->commands, lps->rgb2u, 2, NULL, &global[0], NULL, 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to execute rgb2u kernel: %d!\n", err);
				return 0;
			}
			if((err = clEnqueueNDRangeKernel(lps->commands, lps->rgb2v, 2, NULL, &global[0], NULL, 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to execute rgb2v kernel: %d!\n", err);
				return 0;
			}
		} else {
			if((err = clEnqueueNDRangeKernel(lps->commands, lps->rgb2uv, 2, NULL, &global[0], NULL, 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to execute rgb2uv kernel: %d!\n", err);
				return 0;
			}
		}
		if((err = clFinish(lps->commands)) != CL_SUCCESS) {
			fprintf(stderr, "OpenCL error: failed to wait for commands completion: %d!\n", err);
			return 0;
		}
		size_t origin[3] = { 0, 0, 0 };
		size_t region[3];
		region[0] = surface_width;
		region[1] = surface_height;
		region[2] = 1;
		if((err = clEnqueueReadImage(lps->commands, outputy, CL_TRUE, origin, region, surface_image.pitches[0], 0, (unsigned char*)surface_p+surface_image.offsets[0], 0, NULL, NULL)) != CL_SUCCESS) {
			fprintf(stderr, "OpenCL error: failed to read outputy image: %d!\n", err);
			return 0;
		}
		region[0] /= 2;
		region[1] /= 2;
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			if((err = clEnqueueReadImage(lps->commands, outputu, CL_TRUE, origin, region, surface_image.pitches[1], 0, (unsigned char*)surface_p+surface_image.offsets[1], 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to read outputu image: %d!\n", err);
				return 0;
			}
			if((err = clEnqueueReadImage(lps->commands, outputv, CL_TRUE, origin, region, surface_image.pitches[2], 0, (unsigned char*)surface_p+surface_image.offsets[2], 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to read outputv image: %d!\n", err);
				return 0;
			}
		} else {
			if((err = clEnqueueReadImage(lps->commands, outputuv, CL_TRUE, origin, region, surface_image.pitches[1], 0, (unsigned char*)surface_p+surface_image.offsets[1], 0, NULL, NULL)) != CL_SUCCESS) {
				fprintf(stderr, "OpenCL error: failed to read outputuv image: %d!\n", err);
				return 0;
			}
		}
		clReleaseMemObject(input);
		clReleaseMemObject(outputy);
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			clReleaseMemObject(outputu);
			clReleaseMemObject(outputv);
		} else {
			clReleaseMemObject(outputuv);
		}
#else
		for(i=0;i<surface_height;i++) {
			unsigned char *Y = (unsigned char*)surface_p+surface_image.offsets[0]+i*surface_image.pitches[0];
			unsigned char *U = (unsigned char*)surface_p+surface_image.offsets[1]+i/2*surface_image.pitches[1];
			unsigned char *V;
			if(surface_image.format.fourcc == VA_FOURCC_YV12) {
				V = (unsigned char*)surface_p+surface_image.offsets[2]+i/2*surface_image.pitches[2];
			} else {
				V = (unsigned char*)surface_p+surface_image.offsets[1]+i/2*surface_image.pitches[1]+1;
			}
			for(j=0;j<surface_width/2;j++) {
				rgbtoyuv(rgb[2], rgb[1], rgb[0], *Y, *U, *V);
				rgb += 4; Y++;
				U += uv_step;
				V += uv_step;
				rgbtoy(rgb[2], rgb[1], rgb[0], *Y);
				rgb += 4; Y++;
			}
			i++;
			Y = (unsigned char*)surface_p+surface_image.offsets[0]+i*surface_image.pitches[0];
			for(j=0;j<surface_width;j++) {
				rgbtoy(rgb[2], rgb[1], rgb[0], *Y);
				rgb += 4; Y++;
			}
		}
#endif
	} else {
		for(i=0;i<surface_height;i++) {
			if(!(lps->sgf.modeRDO & RP_PS_MONO8)) {
				char *data = (char*)lps->sgf.bp+i*surface_width*2;
				char * byte_data = surface_p+surface_image.offsets[0]+i*surface_image.pitches[0];
				int bitshift = lps->sgf.curBPP - 8;
#if 0
				int cnt = (lps->sgf.curCX * sizeof(WORD)) / 32;
				_asm{
					mov			esi,		[data]				;
					mov			edi,		[byte_data]			;
					movd		xmm2,		[bitshift]			;
t00:
					movdqa		xmm0,		[esi]		;
					movdqa		xmm1,		[esi+16]	;
					psrlw		xmm0,		xmm2				;
					psrlw		xmm1,		xmm2				;
					packuswb	xmm0,		xmm1				;
					movntdq		[edi],		xmm0				;
					add			edi,		16					;
					add			esi,		32					;

					sub			[cnt],		1					;
					ja			t00								;
				}
#else
				for(int cnt = lps->sgf.curCX; cnt > 0; cnt--) {
					*byte_data = *((uint16_t*)(data)) >> bitshift;
					data += 2;
					byte_data++;
				}
#endif
			} else {
				memcpy(surface_p+surface_image.offsets[0]+i*surface_image.pitches[0], (char*)lps->sgf.bp+i*surface_width, surface_width);
			}
		}
		memset(surface_p + surface_image.offsets[1], 128, surface_image.pitches[1]*surface_image.height/2);
		if(surface_image.format.fourcc == VA_FOURCC_YV12) {
			memset(surface_p + surface_image.offsets[2], 128, surface_image.pitches[2]*surface_image.height/2);
		}
	}
	if(!has_derive) {
		va_status = vaPutImage(va_dpy, surface_id, surface_image.image_id, 0, 0, surface_width, surface_height, 0, 0, surface_width, surface_height);
		CHECK_VASTATUS(va_status, "vaPutImage", vaUnmapBuffer(va_dpy,surface_image.buf);vaDestroyImage(va_dpy,surface_image.image_id);return 0);
	}
	va_status = vaUnmapBuffer(va_dpy,surface_image.buf);
	CHECK_VASTATUS(va_status, "vaUnmapBuffer", vaDestroyImage(va_dpy,surface_image.image_id);return 0);
	va_status = vaDestroyImage(va_dpy,surface_image.image_id);
	CHECK_VASTATUS(va_status, "vaDestroyImage", return 0);
	va_status = vaPutSurface(va_dpy, surface_id, win, 0, 0, surface_width, surface_height, 0, 0, win_width, win_height, 0, 0, VA_FRAME_PICTURE);
	CHECK_VASTATUS(va_status,"vaPutSurface", return 0);
	va_status = vaSyncSurface(va_dpy, surface_id);
	CHECK_VASTATUS(va_status, "vaSyncSurface", return 0);
	return 1;
}
