const sampler_t sampleri =  CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_NONE | CLK_FILTER_NEAREST;
const sampler_t samplerf =  CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_NONE | CLK_FILTER_LINEAR;

kernel void rgb2y(read_only image2d_t in_rgb, write_only image2d_t out_y) {
	const int x = get_global_id(0);
	const int y = get_global_id(1);

	float4 pixel = read_imagef(in_rgb, sampleri, (int2)(x, y));
	uint4 ya;
	ya.w = 16 + 65.481 * pixel.x + 128.553 * pixel.y + 24.966 * pixel.z;
	write_imageui(out_y, (int2)(x, y), ya);
}

#define RGBTOU(R, G, B, U) U = 128 + -37.797 * R - 74.203 * G + 112.000 * B
#define RGBTOV(R, G, B, V) V = 128 + 112.000 * R - 93.786 * G -  18.214 * B

kernel void rgb2uv(read_only image2d_t in_rgb, write_only image2d_t out_uv) {
	const int x = get_global_id(0);
	const int y = get_global_id(1);

	float4 pixel = read_imagef(in_rgb, samplerf, (int2)(x*2+1, y*2+1));
	uint4 uv;
	RGBTOU(pixel.x, pixel.y, pixel.z, uv.x);
	RGBTOV(pixel.x, pixel.y, pixel.z, uv.y);
	write_imageui(out_uv, (int2)(x, y), uv);
}

kernel void rgb2u(read_only image2d_t in_rgb, write_only image2d_t out_u) {
	const int x = get_global_id(0);
	const int y = get_global_id(1);

	float4 pixel = read_imagef(in_rgb, samplerf, (int2)(x*2+1, y*2+1));
	uint4 u;
	RGBTOU(pixel.x, pixel.y, pixel.z, u.w);
	write_imageui(out_u, (int2)(x, y), u);
}

kernel void rgb2v(read_only image2d_t in_rgb, write_only image2d_t out_v) {
	const int x = get_global_id(0);
	const int y = get_global_id(1);

	float4 pixel = read_imagef(in_rgb, samplerf, (int2)(x*2+1, y*2+1));
	uint4 v;
	RGBTOV(pixel.x, pixel.y, pixel.z, v.w);
	write_imageui(out_v, (int2)(x, y), v);
}
